package org.siqisource.agave.orm.condition;

import java.util.Arrays;
import java.util.List;

import org.siqisource.agave.orm.SqlBuilder;
import org.siqisource.agave.orm.expression.BetweenValueExpression;
import org.siqisource.agave.orm.expression.ListValueExpression;
import org.siqisource.agave.orm.expression.NoValueExpression;
import org.siqisource.agave.orm.expression.SingleValueExpression;
import org.siqisource.agave.orm.expression.SqlCompareExpression;
import org.siqisource.agave.utils.NameConverter;

public class SimpleCondition extends Condition {

	public void andIsNotNull(String propertyName) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new NoValueExpression(SqlBuilder.AND, column, " IS NOT NULL ", "");
		this.addExpression(expression);
	}

	public void andIsNull(String propertyName) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new NoValueExpression(SqlBuilder.AND, column, " IS NULL ", "");
		this.addExpression(expression);
	}

	public void andEqual(String propertyName, Object value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " = ", value, "");
		this.addExpression(expression);
	}

	public void andLike(String propertyName, String value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " LIKE ", "%" + value + "%", "");
		this.addExpression(expression);
	}

	public void andNotLike(String propertyName, String value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " NOT LIKE ", "%" + value + "%", "");
		this.addExpression(expression);
	}

	public void andBeginWith(String propertyName, String value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " LIKE ", value + "%", "");
		this.addExpression(expression);
	}

	public void andEndWith(String propertyName, String value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " LIKE ", "%" + value, "");
		this.addExpression(expression);
	}

	public void andNotEqual(String propertyName, Object value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " <> ", value, "");
		this.addExpression(expression);
	}

	public void andGreater(String propertyName, Object value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " > ", value, "");
		this.addExpression(expression);
	}

	public void andNoLess(String propertyName, Object value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " >= ", value, "");
		this.addExpression(expression);
	}

	public void andLess(String propertyName, Object value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " < ", value, "");
		this.addExpression(expression);
	}

	public void andNoGreater(String propertyName, Object value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(SqlBuilder.AND, column, " <= ", value, "");
		this.addExpression(expression);
	}

	public void andBetween(String propertyName, Object beginValue, Object endValue) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new BetweenValueExpression(SqlBuilder.AND, column, beginValue, endValue, "");
		this.addExpression(expression);
	}

	public void andIn(String propertyName, List<Object> value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new ListValueExpression(SqlBuilder.AND, column, " IN ", value, "");
		this.addExpression(expression);
	}

	public void andIn(String propertyName, String values) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new ListValueExpression(SqlBuilder.AND, column, " IN ", Arrays.asList((Object[]) values.split(",")), "");
		this.addExpression(expression);
	}

	public void andNotIn(String propertyName, List<Object> value) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new ListValueExpression(SqlBuilder.AND, column, " NOT IN ", value, "");
		this.addExpression(expression);
	}

	public void andNotIn(String propertyName, String values) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		SqlCompareExpression expression = new ListValueExpression(SqlBuilder.AND, column, " NOT IN ", Arrays.asList((Object[]) values.split(",")), "");
		this.addExpression(expression);
	}

}
