package org.siqisource.agave.security;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.LinkedHashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.core.type.classreading.CachingMetadataReaderFactory;
import org.springframework.core.type.classreading.MetadataReader;
import org.springframework.core.type.classreading.MetadataReaderFactory;
import org.springframework.util.ReflectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

public class FilterChainDefinitionMap extends LinkedHashMap<String, String> {

	private static final long serialVersionUID = 8429436296822324223L;

	public static final Logger logger = LoggerFactory
			.getLogger(FilterChainDefinitionMap.class);

	private ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();

	private MetadataReaderFactory readerFactory = new CachingMetadataReaderFactory(
			this.resourcePatternResolver);

	public void setFilterChainDefinitions(
			ArrayList<String> filterChainDefinitions) {
		for (String filterChainDefinition : filterChainDefinitions) {
			String[] filterChainDefinitionArray = filterChainDefinition
					.split("=");
			this.put(StringUtils.trimWhitespace(filterChainDefinitionArray[0]),
					StringUtils.trimWhitespace(filterChainDefinitionArray[1]));
		}
		scanFilterChainDefinition();
	}

	public void scanFilterChainDefinition() {
		try {
			Resource[] resources = this.resourcePatternResolver
					.getResources("classpath*:/**/*.class");

			for (Resource resource : resources) {
				if (resource.isReadable()) {
					MetadataReader reader = readerFactory
							.getMetadataReader(resource);
					String className = reader.getClassMetadata().getClassName();
					Class<?> clazz = this.resourcePatternResolver
							.getClassLoader().loadClass(className);
					processFilterChain(clazz);
				}
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	private void processFilterChain(Class<?> clazz) throws BeansException {

		ReflectionUtils.doWithMethods(clazz,
				new ReflectionUtils.MethodCallback() {
					public void doWith(Method method) {
						RequestMapping mapping = AnnotationUtils
								.findAnnotation(method, RequestMapping.class);

						Permission permission = AnnotationUtils.findAnnotation(
								method, Permission.class);

						Role role = AnnotationUtils.findAnnotation(method,
								Role.class);

						if (mapping != null) {
							String urls[] = mapping.value();

							if (permission != null) {

								String permissionString = permission.value();
								for (String url : urls) {
									put(url, "perms["+permissionString+"]");
								}
							}

							if (role != null) {
								String[] roleStrings = role.value();
								for (String url : urls) {
									for (String roleString : roleStrings) {
										put(url, "roles["+roleString+"]");
									}
								}
							}

						}
					}
				}, ReflectionUtils.USER_DECLARED_METHODS);

	}

}