package org.siqisource.agave.service;

import java.util.List;

import org.apache.ibatis.session.RowBounds;
import org.siqisource.agave.orm.MybatisMapper;
import org.siqisource.agave.orm.PartitiveFields;
import org.siqisource.agave.orm.condition.Condition;

public abstract class AbstractService<T> implements AgaveService<T> {

	/**
	 * Subclass need tell parent which mapper it is using.
	 * 
	 * @return
	 */
	protected abstract MybatisMapper<T> getMapper();

	/**
	 * insert object to database
	 * 
	 * @param model
	 */
	public void insert(T model) {
		getMapper().insert(model);
	}

	/**
	 * insert object to database batch
	 * 
	 * @param model
	 */
	public void insertBatch(List<T> models) {
		getMapper().insertBatch(models);
	}

	/**
	 * insert some fields of the table
	 * 
	 * @param fields
	 */
	public void insertPartitive(PartitiveFields fields) {
		fields.prepareInsertSql();
		getMapper().insertPartitive(fields);
	}

	/**
	 * read object from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 * @return object
	 */
	public T read(String... id) {
		return getMapper().read(id);
	}
	
	/**
	 * read object from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 * @return object
	 */
	public T read(int... id) {
		return getMapper().read(id);
	}

	/**
	 * count
	 * 
	 * @param condition
	 * @return
	 */
	public int count(Condition condition) {
		return getMapper().count(condition);
	}

	/**
	 * pagination list
	 * 
	 * @param condition
	 * @param rowBounds
	 * @return
	 */
	public List<T> list(Condition condition, RowBounds rowBounds) {
		return getMapper().list(condition, rowBounds);
	}

	/**
	 * list
	 * 
	 * @param condition
	 * @return
	 */
	public List<T> list(Condition condition) {
		return getMapper().list(condition);
	}

	/**
	 * update database
	 * 
	 * @param model
	 */
	public void update(T model) {
		getMapper().update(model);
	}

	/**
	 * update some fields of the table
	 * 
	 * @param fields
	 * @param id
	 */
	public void updatePartitive(PartitiveFields fields, String... id) {
		fields.prepareUpdateSql();
		getMapper().updatePartitive(fields, id);
	}
	
	/**
	 * update some fields of the table
	 * 
	 * @param fields
	 * @param id
	 */
	public void updatePartitive(PartitiveFields fields, int... id) {
		fields.prepareUpdateSql();
		getMapper().updatePartitive(fields, id);
	}

	/**
	 * update batch records
	 * 
	 * @param condition
	 */
	public void updateBatch(PartitiveFields fields, Condition condition) {
		fields.prepareUpdateSql();
		getMapper().updateBatch(fields, condition);
	}
	
	/**
	 * update batch records
	 * 
	 * @param condition
	 */
	public void updateBatch(List<T> models) {
		getMapper().updateBatchList(models);
	}

	/**
	 * delete from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 */
	public void delete(String... id) {
		getMapper().delete(id);
	}
	
	/**
	 * delete from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 */
	public void delete(int... id) {
		getMapper().delete(id);
	}

	/**
	 * delete batch record from database
	 * 
	 * @param condition
	 */
	public void deleteBatch(Condition condition) {
		getMapper().deleteBatch(condition);
	}
}
