package org.siqisource.agave.service;

import java.util.List;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.session.RowBounds;
import org.siqisource.agave.orm.PartitiveFields;
import org.siqisource.agave.orm.condition.Condition;

public interface AgaveService<T> {

	/**
	 * insert object to database
	 * 
	 * @param model
	 */
	public void insert(T model);
	
	/**
	 * insert object to database batch
	 * 
	 * @param model
	 */
	public void insertBatch(List<T> models);

	/**
	 * insert some fields of the table
	 * 
	 * @param fields
	 */
	public void insertPartitive(@Param("fields") PartitiveFields fields);

	/**
	 * read object from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 * @return object
	 */
	public T read(@Param("id") String... id);
	
	/**
	 * read object from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 * @return object
	 */
	public T read(int... id);

	/**
	 * count the querying record
	 * 
	 * @param condition
	 * @return
	 */
	int count(Condition condition);

	/**
	 * list the pagination querying result
	 * 
	 * @param condition
	 * @param rowBounds
	 * @return
	 */
	List<T> list(Condition condition, RowBounds rowBounds);

	/**
	 * list the querying result
	 * 
	 * @param condition
	 * @return
	 */
	List<T> list(Condition condition);

	/**
	 * update database
	 * 
	 * @param model
	 */
	public void update(T model);

	/**
	 * update some fields of the table
	 * 
	 * @param fields
	 * @param id
	 */
	public void updatePartitive(PartitiveFields fields, @Param("id") String... id);

	/**
	 * update batch records
	 * 
	 * @param condition
	 */
	public void updateBatch(PartitiveFields fields, @Param("condition") Condition condition);

	/**
	 * delete from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 */
	public void delete(String... id);
	
	/**
	 * delete from database
	 * 
	 * @param id
	 *            primary key(s) of the table
	 */
	public void delete(int... id);

	/**
	 * delete batch record from database
	 * 
	 * @param condition
	 */
	public void deleteBatch(Condition condition);
}
